@@ -13,8 +13,8 @@ import (
 	"text/template"
 
 	"github.com/containers/image/pkg/sysregistriesv2"
-	"github.com/coreos/ignition/config/util"
-	igntypes "github.com/coreos/ignition/config/v2_2/types"
+	ignutil "github.com/coreos/ignition/v2/config/util"
+	igntypes "github.com/coreos/ignition/v2/config/v3_1/types"
 	configv1 "github.com/openshift/api/config/v1"
 	"github.com/pkg/errors"
 	"github.com/sirupsen/logrus"
@@ -22,6 +22,8 @@ import (
 	"github.com/openshift/installer/data"
 	"github.com/openshift/installer/pkg/asset"
 	"github.com/openshift/installer/pkg/asset/ignition"
+	"github.com/openshift/installer/pkg/asset/ignition/bootstrap/baremetal"
+	"github.com/openshift/installer/pkg/asset/ignition/bootstrap/vsphere"
 	"github.com/openshift/installer/pkg/asset/installconfig"
 	"github.com/openshift/installer/pkg/asset/kubeconfig"
 	"github.com/openshift/installer/pkg/asset/machines"
@@ -30,6 +32,8 @@ import (
 	"github.com/openshift/installer/pkg/asset/rhcos"
 	"github.com/openshift/installer/pkg/asset/tls"
 	"github.com/openshift/installer/pkg/types"
+	baremetaltypes "github.com/openshift/installer/pkg/types/baremetal"
+	vspheretypes "github.com/openshift/installer/pkg/types/vsphere"
 )
 
 const (
@@ -49,6 +53,15 @@ type bootstrapTemplateData struct {
 	Proxy                 *configv1.ProxyStatus
 	Registries            []sysregistriesv2.Registry
 	BootImage             string
+	ClusterDomain         string
+	PlatformData          platformTemplateData
+}
+
+// platformTemplateData is the data to use to replace values in bootstrap
+// template files that are specific to one platform.
+type platformTemplateData struct {
+	BareMetal *baremetal.TemplateData
+	VSphere   *vsphere.TemplateData
 }
 
 // Bootstrap is an asset that generates the ignition config for bootstrap nodes.
@@ -77,6 +90,7 @@ func (a *Bootstrap) Dependencies() []asset.Asset {
 		&tls.AggregatorClientCertKey{},
 		&tls.AggregatorSignerCertKey{},
 		&tls.APIServerProxyCertKey{},
+		&tls.BootstrapSSHKeyPair{},
 		&tls.EtcdCABundle{},
 		&tls.EtcdMetricCABundle{},
 		&tls.EtcdMetricSignerCertKey{},
@@ -122,7 +136,8 @@ func (a *Bootstrap) Generate(dependencies asset.Parents) error {
 	proxy := &manifests.Proxy{}
 	releaseImage := &releaseimage.Image{}
 	rhcosImage := new(rhcos.Image)
-	dependencies.Get(installConfig, proxy, releaseImage, rhcosImage)
+	bootstrapSSHKeyPair := &tls.BootstrapSSHKeyPair{}
+	dependencies.Get(installConfig, proxy, releaseImage, rhcosImage, bootstrapSSHKeyPair)
 
 	templateData, err := a.getTemplateData(installConfig.Config, releaseImage.PullSpec, installConfig.Config.ImageContentSources, proxy.Config, rhcosImage)
 
@@ -171,10 +186,13 @@ func (a *Bootstrap) Generate(dependencies asset.Parents) error {
 
 	a.Config.Passwd.Users = append(
 		a.Config.Passwd.Users,
-		igntypes.PasswdUser{Name: "core", SSHAuthorizedKeys: []igntypes.SSHAuthorizedKey{igntypes.SSHAuthorizedKey(installConfig.Config.SSHKey)}},
+		igntypes.PasswdUser{Name: "core", SSHAuthorizedKeys: []igntypes.SSHAuthorizedKey{
+			igntypes.SSHAuthorizedKey(installConfig.Config.SSHKey),
+			igntypes.SSHAuthorizedKey(string(bootstrapSSHKeyPair.Public())),
+		}},
 	)
 
-	data, err := json.Marshal(a.Config)
+	data, err := ignition.Marshal(a.Config)
 	if err != nil {
 		return errors.Wrap(err, "failed to Marshal Ignition config")
 	}
@@ -222,6 +240,16 @@ func (a *Bootstrap) getTemplateData(installConfig *types.InstallConfig, releaseI
 		registries = append(registries, registry)
 	}
 
+	// Generate platform-specific bootstrap data
+	var platformData platformTemplateData
+
+	switch installConfig.Platform.Name() {
+	case baremetaltypes.Name:
+		platformData.BareMetal = baremetal.GetTemplateData(installConfig.Platform.BareMetal, installConfig.MachineNetwork)
+	case vspheretypes.Name:
+		platformData.VSphere = vsphere.GetTemplateData(installConfig.Platform.VSphere)
+	}
+
 	return &bootstrapTemplateData{
 		AdditionalTrustBundle: installConfig.AdditionalTrustBundle,
 		FIPS:                  installConfig.FIPS,
@@ -231,6 +259,8 @@ func (a *Bootstrap) getTemplateData(installConfig *types.InstallConfig, releaseI
 		Proxy:                 &proxy.Status,
 		Registries:            registries,
 		BootImage:             string(*rhcosImage),
+		ClusterDomain:         installConfig.ClusterDomain(),
+		PlatformData:          platformData,
 	}, nil
 }
 
@@ -285,8 +315,12 @@ func (a *Bootstrap) addStorageFiles(base string, uri string, templateData *boots
 		mode = 0600
 	}
 	ign := ignition.FileFromBytes(strings.TrimSuffix(base, ".template"), "root", mode, data)
-	ign.Append = appendToFile
-	a.Config.Storage.Files = append(a.Config.Storage.Files, ign)
+	if appendToFile {
+		ignition.ConvertToAppendix(&ign)
+	}
+
+	// Replace files that already exist in the slice with ones added later, otherwise append them
+	a.Config.Storage.Files = replaceOrAppend(a.Config.Storage.Files, ign)
 
 	return nil
 }
@@ -299,9 +333,10 @@ func (a *Bootstrap) addSystemdUnits(uri string, templateData *bootstrapTemplateD
 		"systemd-journal-gatewayd.socket": {},
 		"approve-csr.service":             {},
 		// baremetal & openstack platform services
-		"keepalived.service": {},
-		"coredns.service":    {},
-		"ironic.service":     {},
+		"keepalived.service":        {},
+		"coredns.service":           {},
+		"ironic.service":            {},
+		"master-bmh-update.service": {},
 	}
 
 	directory, err := data.Assets.Open(uri)
@@ -342,7 +377,7 @@ func (a *Bootstrap) addSystemdUnits(uri string, templateData *bootstrapTemplateD
 				return err
 			}
 
-			dropins := []igntypes.SystemdDropin{}
+			dropins := []igntypes.Dropin{}
 			for _, childInfo := range children {
 				file, err := data.Assets.Open(path.Join(dir, childInfo.Name()))
 				if err != nil {
@@ -355,9 +390,9 @@ func (a *Bootstrap) addSystemdUnits(uri string, templateData *bootstrapTemplateD
 					return err
 				}
 
-				dropins = append(dropins, igntypes.SystemdDropin{
+				dropins = append(dropins, igntypes.Dropin{
 					Name:     childName,
-					Contents: string(contents),
+					Contents: ignutil.StrToPtr(string(contents)),
 				})
 			}
 
@@ -367,7 +402,7 @@ func (a *Bootstrap) addSystemdUnits(uri string, templateData *bootstrapTemplateD
 				Dropins: dropins,
 			}
 			if _, ok := enabled[name]; ok {
-				unit.Enabled = util.BoolToPtr(true)
+				unit.Enabled = ignutil.BoolToPtr(true)
 			}
 			a.Config.Systemd.Units = append(a.Config.Systemd.Units, unit)
 		} else {
@@ -378,10 +413,10 @@ func (a *Bootstrap) addSystemdUnits(uri string, templateData *bootstrapTemplateD
 
 			unit := igntypes.Unit{
 				Name:     name,
-				Contents: string(contents),
+				Contents: ignutil.StrToPtr(string(contents)),
 			}
 			if _, ok := enabled[name]; ok {
-				unit.Enabled = util.BoolToPtr(true)
+				unit.Enabled = ignutil.BoolToPtr(true)
 			}
 			a.Config.Systemd.Units = append(a.Config.Systemd.Units, unit)
 		}
@@ -423,7 +458,11 @@ func (a *Bootstrap) addParentFiles(dependencies asset.Parents) {
 		&machines.Worker{},
 	} {
 		dependencies.Get(asset)
-		a.Config.Storage.Files = append(a.Config.Storage.Files, ignition.FilesFromAsset(rootDir, "root", 0644, asset)...)
+
+		// Replace files that already exist in the slice with ones added later, otherwise append them
+		for _, file := range ignition.FilesFromAsset(rootDir, "root", 0644, asset) {
+			a.Config.Storage.Files = replaceOrAppend(a.Config.Storage.Files, file)
+		}
 	}
 
 	// These files are all added with mode 0600; use for secret keys and the like.
@@ -472,12 +511,27 @@ func (a *Bootstrap) addParentFiles(dependencies asset.Parents) {
 		&tls.JournalCertKey{},
 	} {
 		dependencies.Get(asset)
-		a.Config.Storage.Files = append(a.Config.Storage.Files, ignition.FilesFromAsset(rootDir, "root", 0600, asset)...)
+
+		// Replace files that already exist in the slice with ones added later, otherwise append them
+		for _, file := range ignition.FilesFromAsset(rootDir, "root", 0600, asset) {
+			a.Config.Storage.Files = replaceOrAppend(a.Config.Storage.Files, file)
+		}
 	}
 
 	rootCA := &tls.RootCA{}
 	dependencies.Get(rootCA)
-	a.Config.Storage.Files = append(a.Config.Storage.Files, ignition.FileFromBytes(filepath.Join(rootDir, rootCA.CertFile().Filename), "root", 0644, rootCA.Cert()))
+	a.Config.Storage.Files = replaceOrAppend(a.Config.Storage.Files, ignition.FileFromBytes(filepath.Join(rootDir, rootCA.CertFile().Filename), "root", 0644, rootCA.Cert()))
+}
+
+func replaceOrAppend(files []igntypes.File, file igntypes.File) []igntypes.File {
+	for i, f := range files {
+		if f.Node.Path == file.Node.Path {
+			files[i] = file
+			return files
+		}
+	}
+	files = append(files, file)
+	return files
 }
 
 func applyTemplateData(template *template.Template, templateData interface{}) string {
